#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # PCA (Principal Component Analysis)
# 
# This notebook shows how to plot a PCA with sciki-learn and statsmodels, with or without normalization.

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[2]:


import matplotlib.pyplot as plt
plt.style.use('ggplot')


# In[3]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# More about [PCA](https://en.wikipedia.org/wiki/Principal_component_analysis): [Implementing a Principal Component Analysis (PCA) in Python step by step](http://sebastianraschka.com/Articles/2014_pca_step_by_step.html).

# ## Download data

# In[4]:


import pyensae.datasource
pyensae.datasource.download_data("auto-mpg.data", url="https://archive.ics.uci.edu/ml/machine-learning-databases/auto-mpg/")


# In[5]:


import pandas
df = pandas.read_fwf("auto-mpg.data", encoding="utf-8",
                    names="mpg cylinders displacement horsepower weight acceleration year origin name".split())
df["name"] = df["name"].apply(lambda s : s.strip(' "'))
df.head()


# In[6]:


df.dtypes


# We remove missing values:

# In[7]:


df[df.horsepower == "?"]


# In[8]:


final = df[df.horsepower != '?'].copy()
final["horsepower"] = final["horsepower"].astype(float)


# In[9]:


final.to_csv("auto-mpg.data.csv", sep="\t", index=False, encoding="utf-8")


# In[10]:


final.shape


# ## PCA with scikit-learn

# In[11]:


from sklearn.decomposition import PCA
X = final[df.columns[1:-1]]
Y = final["mpg"]
pca = PCA(n_components=2)
pca.fit(X)


# In[12]:


out = pca.transform(X)
out[:5]


# In[13]:


pca.explained_variance_ratio_, pca.noise_variance_


# In[14]:


import matplotlib.pyplot as plt
plt.plot(out[:,0], out[:,1], ".");


# In[15]:


pca.components_ 


# ## PCA with scikit-learn and normalization

# In[16]:


from sklearn.decomposition import PCA
from sklearn.preprocessing import Normalizer
from sklearn.pipeline import Pipeline

normpca = Pipeline([('norm', Normalizer()), ('pca', PCA(n_components=2))])
normpca.fit(X)


# In[17]:


out = normpca.transform(X)
out[:5]


# In[18]:


normpca.named_steps['pca'].explained_variance_ratio_, normpca.named_steps['pca'].noise_variance_


# In[19]:


import matplotlib.pyplot as plt
plt.plot(out[:,0], out[:,1], ".");


# In[20]:


normpca.named_steps['pca'].components_


# ## PCA with statsmodels

# In[21]:


from statsmodels.sandbox.tools import pca
xred, fact, eva, eve = pca(X, keepdim=2, normalize=False)


# In[22]:


fact[:5]


# In[23]:


eva


# In[24]:


eve


# In[25]:


plt.plot(fact[:,0], fact[:,1], ".");


# ## PCA with statsmodels and normalization

# In[26]:


from statsmodels.sandbox.tools import pca
from sklearn.preprocessing import normalize
X_norm = normalize(X)
xred, fact, eva, eve = pca(X_norm, keepdim=2, normalize=True)


# In[27]:


eva


# In[28]:


eve


# In[29]:


plt.plot(fact[:,0], fact[:,1], ".");


# In[30]:





# In[31]: